//-----------------------------------------------------------------------------
Unit UCoreTriangleAPI;
//-----------------------------------------------------------------------------

Interface

{$DEFINE WIN_CONSOLE}

Uses Windows,
    SysUtils,
    {$ifdef WIN_CONSOLE}
    UExConsoleForm,
    {$endif}
    UExStrings;

Const cMaxLine  = 63;
      cMaxWidth = 127;

//-----------------------------------------------------------------------------
Type
 TReal      = Double;
 TInteger   = Integer;

//-----------------------------------------------------------------------------
 PRealArray    = ^TRealArray;
 TRealArray    = Array [0..0] Of TReal;

//-----------------------------------------------------------------------------
 PIntegerArray = ^TIntegerArray;
 TIntegerArray = Array [0..0] Of TInteger;

//-----------------------------------------------------------------------------
 PTriangulateIO =^TTriangulateIO;
 TTriangulateIO = Record
  PointList                  : PRealArray;
  PointAttributeList         : PRealArray;
  PointMarkerList            : PIntegerArray;
  NumberOfPoints             : TInteger;
  NumberOfPointAttributes    : TInteger;

  TriangleList               : PIntegerArray;
  TriangleAttributeList      : PRealArray;
  TriangleAreaList           : PRealArray;
  NeighborList               : PIntegerArray;
  NumberOfTriangles          : TInteger;
  NumberOfCorners            : TInteger;
  NumberOfTriangleAttributes : TInteger;

  SegmentList                : PIntegerArray;
  SegmentMarkerList          : PIntegerArray;
  NumberOfSegments           : TInteger;

  HoleList                   : PRealArray;
  NumberOfHoles              : TInteger;

  RegionList                 : PRealArray;
  NumberOfRegions            : TInteger;

  EdgeList                   : PIntegerArray;
  EdgeMarkerList             : PIntegerArray;
  NormList                   : PRealArray;
  NumberOfEdges              : TInteger;
 End;

//-----------------------------------------------------------------------------
TStringArray  = Array [0..cMaxLine] Of PChar;

//-----------------------------------------------------------------------------
TTriangulate    = procedure(
                             Switches:PChar;
                             InData,
                             OutData,
                             VorOut:PTriangulateIO);cdecl;
//-----------------------------------------------------------------------------
TFreeTriangleIO = procedure (Data:PTriangulateIO);cdecl;

//-----------------------------------------------------------------------------
Function _triangulate(
                             Switches:PChar;
                             InData,
                             OutData,
                             VorOut:PTriangulateIO):Integer;cdecl;external
                             'Triangle.DLL';

//-----------------------------------------------------------------------------
procedure _freetriangleio(Data:PTriangulateIO);cdecl;external 'Triangle.DLL';

//-----------------------------------------------------------------------------
procedure _openiobuffer(cMaxLine,cIOWidth:Integer);cdecl;external 'Triangle.DLL';
procedure _freeiobuffer;cdecl;external 'Triangle.DLL';
procedure _cleariobuffer;cdecl;external 'Triangle.DLL';
Function  _iobufferlines:Integer;cdecl;external 'Triangle.DLL';
Function  _iobuffertextwidth:Integer;cdecl;external 'Triangle.DLL';
Function  _getiobuffer (Num:Integer;Data:PChar):Integer;cdecl;external 'Triangle.DLL';

// -------------------------------------------------------------------
Function ExecTriangulate(
         Switches:PChar;
         Var InData,OutData,
         VorOut:PTriangulateIO):LongInt;
//-----------------------------------------------------------------------------
Function TriangulateLoadDLL(
         Switches:PChar;
         Var InData,OutData,
         VorOut:PTriangulateIO):LongInt;

//-----------------------------------------------------------------------------
{$ifdef WIN_CONSOLE}
Procedure WriteStringsIO(Port:TfrmExConsole);

Procedure Report(Port:TfrmExConsole;
                 IO: TTriangulateIO;
                 Markers,
                 ReportTriangles,
                 ReportNeighbors,
                 ReportSegments,
                 ReportEdges,
                 ReportNorms:Boolean);

{$else}
Procedure WriteStringsIO;

Procedure Report(
                 IO: TTriangulateIO;
                 Markers,
                 ReportTriangles,
                 ReportNeighbors,
                 ReportSegments,
                 ReportEdges,
                 ReportNorms:Boolean);
{$endif}
//-----------------------------------------------------------------------------
Procedure _dlpFreeTriangleIO(Var Data:PTriangulateIO);
//-----------------------------------------------------------------------------
Procedure _InitTriangleIO(Var Data:PTriangulateIO);

//-----------------------------------------------------------------------------
Function NewRealArray(Var Data:PRealArray;Size:Integer):Boolean;
// -------------------------------------------------------------------
Procedure FreeRealArray(Var Data:PRealArray;Dummy:Integer);
//-----------------------------------------------------------------------------
Procedure SetRealArray(Buffer:PRealArray; index:Integer;Data:TReal);


//-----------------------------------------------------------------------------
Function NewIntegerArray(Var Data:PIntegerArray;Size:Integer):Boolean;
// -------------------------------------------------------------------
Procedure FreeIntegerArray(Var Data:PIntegerArray;Dummy:Integer);
//-----------------------------------------------------------------------------
Procedure SetIntegerArray(Buffer:PIntegerArray; index:Integer;Data:TInteger);

//-----------------------------------------------------------------------------
Implementation

// -------------------------------------------------------------------
{$ifdef WIN_CONSOLE}
Procedure WriteStringsIO(Port:TfrmExConsole);
 Var i,m,k:Integer;S,h:String;
     X:PChar;
 Begin
  m:=_iobufferlines;
  X:=StrAlloc(_iobuffertextwidth);
  For i:=0 To m-2 Do Begin
   strcopy(x,'');
   strcopy(X,'\0');
   _getiobuffer(i,X);
   S:=String(X);
   k:=Pos(#10,S);
   While k>0 do Begin
    H:=Copy(S,1,k-1);
    Port.Echo(H);
    Delete(S,1,k);
    k:=Pos(#10,S);
   End;
   Port.EchoEnd(S);
  End;
 End;
{$else}

Procedure WriteStringsIO;
 Var i,m:Integer;S:String;
     X:PChar;
 Begin
  m:=_iobufferlines;
  X:=StrAlloc(_iobuffertextwidth);
  For i:=0 To m-1 Do Begin
   strcopy(x,'');
   _getiobuffer(i,X);
   S:=String(X);
   S:=ReplaceChar(#13,' ',S);
   S:=ReplaceChar(#10,' ',S);
   S:=TrimRight(S);
   Writeln(S);
  End;
 End;
{$endif}
//-----------------------------------------------------------------------------
Procedure FreeRealArray(Var Data:PRealArray;Dummy:Integer);
 Begin
  If Data <> NIL Then Begin
   Freemem(Data);
  End;
 End;

//-----------------------------------------------------------------------------
Procedure FreeIntegerArray(Var Data:PIntegerArray;Dummy:Integer);
 Begin
  If Data <> NIL Then Begin
   Freemem(Data);
  End;
 End;
//-----------------------------------------------------------------------------
Procedure _dlpFreeTriangleIO(Var Data:PTriangulateIO);
 Begin
  FreeRealArray(Data^.PointList, 0);
  FreeRealArray(Data^.PointAttributeList, 0);
  FreeIntegerArray(Data^.PointMarkerlist, 0);
  FreeRealArray(Data^.Regionlist, 0);
  FreeIntegerArray(Data^.Edgelist, 0);
  FreeIntegerArray(Data^.SegmentList, 0);
  FreeIntegerArray(Data^.SegmentMarkerList, 0);
  Data^.NumberOfSegments:=0;
  FreeRealArray(Data^.Holelist, 0);
  Data^.NumberOfHoles:=0;
  FreeRealArray(Data^.Normlist, 0);
  FreeIntegerArray(Data^.TriangleList, 0);
  FreeRealArray(Data^.triangleAttributeList, 0);
 End;
//-----------------------------------------------------------------------------
Procedure _InitTriangleIO(Var Data:PTriangulateIO);
 Begin
  With Data^ Do Begin
    PointList                  :=NIL;
    PointAttributeList         :=NIL;
    PointMarkerList            :=NIL;
    NumberOfPoints             :=0;
    NumberOfPointAttributes    :=0;

    TriangleList               :=NIL;
    TriangleAttributeList      :=NIL;
    TriangleAreaList           :=NIL;
    NeighborList               :=NIL;
    NumberOfTriangles          :=0;
    NumberOfCorners            :=0;
    NumberOfTriangleAttributes :=0;

    SegmentList                :=NIL;
    SegmentMarkerList          :=NIL;
    NumberOfSegments           :=0;

    HoleList                   :=NIL;
    NumberOfHoles              :=0;

    RegionList                 :=NIL;
    NumberOfRegions            :=0;

    EdgeList                   :=NIL;
    EdgeMarkerList             :=NIL;
    NormList                   :=NIL;
    NumberOfEdges              :=0;
 End;
End;
//-----------------------------------------------------------------------------
Function NewRealArray(Var Data:PRealArray;Size:Integer):Boolean;
 Begin
  Result:=False;
  Try
   GetMem(Data,Size*SizeOf(TReal));
   Result:=True;
  Except
   Result:=False;
  End;
 End;
//-----------------------------------------------------------------------------
Function NewIntegerArray(Var Data:PIntegerArray;Size:Integer):Boolean;
 Begin
  Result:=False;
  Try
   GetMem(Data,Size*SizeOf(TInteger));
   Result:=True;
  Except
   Result:=False;
  End;
 End;
//-----------------------------------------------------------------------------
Procedure SetRealArray(Buffer:PRealArray; index:Integer;Data:TReal);
 Begin
  Buffer^[Index]:=Data;
 End;

//-----------------------------------------------------------------------------
Procedure SetIntegerArray(Buffer:PIntegerArray; index:Integer;Data:TInteger);
 Begin
  Buffer^[Index]:=Data;
 End;

//-----------------------------------------------------------------------------
Function TriangulateLoadDLL(
         Switches:PChar;
         Var InData,OutData,
         VorOut:PTriangulateIO):LongInt;

Var  Handle: THandle;
     Triangulate: TTriangulate;
     FreeTreangleio:TFreeTriangleIO;
Begin
  	Handle := LoadLibrary('Triangle.DLL');
  	if Handle <> 0 then begin
       	 @Triangulate := GetProcAddress(Handle, '_triangulate');
         @FreeTreangleio:=GetProcAddress(Handle, '_freetriangleio');
    	 If @Triangulate <> NIL then Begin

       	     Triangulate(Switches,InData,OutData,VorOut);
          End;
          FreeTreangleIO(OutData);
          FreeTreangleIO(VorOut);
    	  FreeLibrary(Handle);
 	End;
End;

//-----------------------------------------------------------------------------
Function ExecTriangulate(
         Switches:PChar;
         Var InData,OutData,
         VorOut:PTriangulateIO):LongInt;
 Begin
  _cleariobuffer;
  Result:=_Triangulate(Switches,InData,OutData,VorOut);
 End;

//-----------------------------------------------------------------------------
{$ifdef WIN_CONSOLE}
Procedure Report(Port:TFrmExConsole;
                 IO: TTriangulateIO;
                 Markers,
                 ReportTriangles,
                 ReportNeighbors,
                 ReportSegments,
                 ReportEdges,
                 ReportNorms:Boolean);
Var i,j:TInteger;
Begin
  For i:=0 To IO.NumberOfPoints-1 Do Begin
    Port.EchoEnd(Format('Point %4d:',[i]));
    For j:=0 To 1 Do
      Port.EchoEnd(Format('  %2.6F', [IO.PointList^[i * 2 + j]]));

    If (IO.NumberOfPointAttributes > 0) Then Begin
      Port.EchoEnd('  Attributes');
      For j:=0 To IO.NumberOfPointAttributes-1 Do
       Port.EchoEnd(Format('  %2.6F',
             [IO.PointAttributeList^[i * IO.NumberOfPointAttributes + j]]));
     End;
    If Markers Then
      Port.EchoEnd(Format(' Marker %d',[IO.PointMarkerList^[i]]));

    Port.Echo('');
  End;

  Port.Echo('');

  If (ReportTriangles OR ReportNeighbors) Then Begin
    For i:=0 To IO.NumberOfTriangles-1 Do Begin
      If (ReportTriangles) Then Begin
        Port.EchoEnd(Format('Triangle %4d points:',[i]));
        For j:=0 To IO.NumberOfCorners-1 Do
          Port.EchoEnd(Format('  %4d', [IO.Trianglelist^[i * IO.NumberOfCorners + j]]));
        If IO.NumberOfTriangleAttributes > 0 Then Begin
          Port.EchoEnd('   Attributes');
          For j:=0 To IO.NumberOfTriangleAttributes-1 Do
          Port.EchoEnd(Format('  %2.6F', [IO.TriangleAttributeList^[i *
                                         IO.NumberofTriangleAttributes + j]]));
        End;
        If (ReportNeighbors) Then Begin
          Port.EchoEnd(Format('Triangle %4d neighbors:',[ i]));
          For j:=0 to 2 Do
           Port.EchoEnd(Format('   %4d',[IO.NeighborList^[i * 3 + j]]));
       End;
      End;
      Port.Echo('');
    End; // for i
    Port.Echo('');;
  End;

  If (ReportSegments) Then Begin
    For i:=0 To IO.NumberOfSegments-1 Do Begin
      Port.EchoEnd(Format('Segment %4d Points:',[ i]));
      For j:=0 To 1 Do
        Port.EchoEnd(Format('  %4d', [IO.SegmentList^[i * 2 + j]]));
      If Markers Then
        Port.Echo(Format('  Marker %d', [IO.SegmentMarkerList^[i]]))
      Else
        Port.Echo('');
   End;
   Port.Echo('');
  End;

  If ReportEdges Then Begin
    For i:=0 To IO.NumberOfEdges-1 Do Begin
      Port.EchoEnd(Format('Edge %4d Points:',[ i]));
      For j:=0 To 1 Do
        Port.EchoEnd(Format('  %4d', [IO.EdgeList^[i * 2 + j]]));

      If (ReportNorms AND (IO.EdgeList^[i * 2 + 1]=-1)) Then Begin
        For j:=0 To 1 Do
          Port.EchoEnd(Format('  %2.6F', [IO.NormList^[i * 2 + j]]));

      End;
      If (Markers) Then
        Port.Echo(Format('  Marker %d', [IO.EdgeMarkerList^[i]]))
      Else
        Port.Echo('');

    End;
    Port.Echo('');
  End;
End;
{$else}
Procedure Report(
                 IO: TTriangulateIO;
                 Markers,
                 ReportTriangles,
                 ReportNeighbors,
                 ReportSegments,
                 ReportEdges,
                 ReportNorms:Boolean);
Var i,j:TInteger;
Begin
  For i:=0 To IO.NumberOfPoints-1 Do Begin
    Write(Format('Point %4d:',[i]));
    For j:=0 To 1 Do
      Write(Format('  %2.6F', [IO.PointList^[i * 2 + j]]));

    If (IO.NumberOfPointAttributes > 0) Then Begin
      Write('  Attributes');
      For j:=0 To IO.NumberOfPointAttributes-1 Do
       Write(Format('  %2.6F',
             [IO.PointAttributeList^[i * IO.NumberOfPointAttributes + j]]));
     End;
    If Markers Then
      Write(Format(' Marker %d',[IO.PointMarkerList^[i]]));

    Writeln('');
  End;

  Writeln('');

  If (ReportTriangles OR ReportNeighbors) Then Begin
    For i:=0 To IO.NumberOfTriangles-1 Do Begin
      If (ReportTriangles) Then Begin
        Write(Format('Triangle %4d points:',[i]));
        For j:=0 To IO.NumberOfCorners-1 Do
          Write(Format('  %4d', [IO.Trianglelist^[i * IO.NumberOfCorners + j]]));
        If IO.NumberOfTriangleAttributes > 0 Then Begin
          Write('   Attributes');
          For j:=0 To IO.NumberOfTriangleAttributes-1 Do
          Write(Format('  %2.6F', [IO.TriangleAttributeList^[i *
                                         IO.NumberofTriangleAttributes + j]]));
        End;
        If (ReportNeighbors) Then Begin
          Write(Format('Triangle %4d neighbors:',[ i]));
          For j:=0 to 2 Do
           Write(Format('   %4d',[IO.NeighborList^[i * 3 + j]]));
       End;
      End;
      Writeln('');
    End; // for i
    Writeln('');;
  End;

  If (ReportSegments) Then Begin
    For i:=0 To IO.NumberOfSegments-1 Do Begin
      Write(Format('Segment %4d Points:',[ i]));
      For j:=0 To 1 Do
        Write(Format('  %4d', [IO.SegmentList^[i * 2 + j]]));
      If Markers Then
        Writeln(Format('  Marker %d', [IO.SegmentMarkerList^[i]]))
      Else
        Writeln('');
   End;
   Writeln('');
  End;

  If ReportEdges Then Begin
    For i:=0 To IO.NumberOfEdges-1 Do Begin
      Write(Format('Edge %4d Points:',[ i]));
      For j:=0 To 1 Do
        Write(Format('  %4d', [IO.EdgeList^[i * 2 + j]]));

      If (ReportNorms AND (IO.EdgeList^[i * 2 + 1]=-1)) Then Begin
        For j:=0 To 1 Do
          Write(Format('  %2.6F', [IO.NormList^[i * 2 + j]]));

      End;
      If (Markers) Then
        Writeln(Format('  Marker %d', [IO.EdgeMarkerList^[i]]))
      Else
        Writeln('');

    End;
    Writeln('');
  End;
End;
{$endif}

//-----------------------------------------------------------------------------
end.
//-----------------------------------------------------------------------------
